/*
 * All Rights Reserved, Copyright (C) 2003, Hitachi Software Engineering Co., Ltd.
 */
/*Modification for Fedora Core4 and many clean up Yuichi Nakamura <ynakam@gwu.edu>*/
/*Modification for Fedora Core2 Takefumi Onabuta <onabuta@selinux.gr.jp>*/
/* $Id: initial_policy.c,v 1.18 2005/06/24 16:42:18 ynakam Exp $ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <selinux/context.h>
#include "initial_policy.h"
#include "security_class.h"
#include "file_label.h"
#include "common.h"
#include "global.h"

char *attribute_list[MAX_ATTR];

/**
 *  @name:	include_file
 *  @about:	output the content of "filename" to "outfp"
 *  @args:	filename (char *) -> filename
 *  @args:	outfp (FILE *) -> output file descripter
 *  @return:	return 0 on success, return -1 in failure.
 */
int
include_file(char *filename, FILE *outfp)
{
	FILE *fp;
	char buf[BUFSIZE];

	if ((fp=fopen(filename,"r")) == NULL)
	{
		fprintf(stderr, "file open error %s\n", filename);
		return -1;
	}

	fprintf(outfp, "#start of file:%s\n\n", filename);
	while (fgets(buf, sizeof(buf), fp) != NULL)
	{
		fprintf(outfp, "%s", buf);
	}
	fprintf(outfp, "#end of file:%s\n\n", filename);

	fclose(fp);

	return 0;
}

/**
 *  @name:	declare_attributes
 *  @about:	declare default attribute
 *  @args:	outfp (FILE *) -> output file descripter
 *  @return:	none
 */
void
declare_attributes(FILE *outfp){	
  FILE *fp;
  char buf[BUFSIZE];  
  char *work;
  char *identifier;
  char *attr;
  char *filename = get_base_policy_files()->attribute_te;
  int i=0;
  int attr_num=0;
  /*declare*/
  include_file(filename, outfp);
  
  /*construct attribute_list*/
  for(i = 0; i<MAX_ATTR;i++)
    attribute_list[i]=NULL;
 
  if ((fp=fopen(filename,"r")) == NULL){
    fprintf(stderr, "file open error %s\n", filename);
    exit(1);
  }
  while (fgets(buf, sizeof(buf), fp) != NULL){	
    work = strdup(buf);      
    identifier = get_nth_tok(work," \t", 1);
        if(identifier == NULL){
	  free(work);
	  continue;
	}
	if(strcmp(identifier,"attribute") == 0){
	  free(work);
	  work = strdup(buf);
	  attr = get_nth_tok(work," \t", 2);
	  chop_str(attr," \t\n;");
	  attribute_list[attr_num] = strdup(attr);
	  attr_num++;
	  if(attr_num == MAX_ATTR){
	    fprintf(stderr,"Too many attributes are declared. Modify MAX_ATTR and recompile\n");
	    exit(1);
	  }
	  free(work);
	}
  }  

  
  fclose(fp); 
}

/**
 *  @name:	declare_initial_types
 *  @about:	declare default types used in types.te and genfs_contexts
 *  @args:	outfp (FILE *) -> output file pointer
 *  @return:	none
 */
void
declare_initial_types(FILE *outfp)
{	
	FILE *fp;
	char buf[BUFSIZE];
	char *types_te;
	char *work;
	char *identifier;
	char *type;

	/*
	  output contents in types.te
	  register types in types.te to label table
	 */
	types_te = get_base_policy_files()->types_te;

	if ((fp=fopen(types_te,"r")) == NULL)
	  {	  
	    fprintf(stderr, "file open error %s\n", types_te);
	    exit(1); 
	  }
	
	while (fgets(buf, sizeof(buf), fp) != NULL)
	  {
	    /*output contents in types.te*/
	    fprintf(outfp, "%s", buf);
	    /*register types in types.te to label table*/
	    work = strdup(buf);
	    identifier = get_nth_tok(work," \t", 1);
	    if(identifier == NULL)
	      {
		free(work);
		continue;
	      }
	    if(strcmp(identifier,"type") == 0)
	      {
		free(work);
		work = strdup(buf);
		type = get_nth_tok(work," \t", 2);
	
		if(type != NULL)
		  {
		    type = get_nth_tok(type,",",1);
		  }
		if(type!=NULL)
		  {
		    if(register_label_table(type) == -2)
		      printf("In types.te:type %s is already registered\n", type);
		  }
	      }
	    free(work);
	  }
	fclose(fp);
	return;
}



/**
 *  @name:	declare_initial_users
 *  @about:	declare default users
 *  @args:	outfp (FILE *) -> output file descripter
 *  @return:	none
 */
void
declare_initial_users(FILE *outfp)
{
	fprintf(outfp, "user system_u roles system_r;\n\n\n");
}

/**
 *  @name:	declare_initial_constrains
 *  @about:	print dummy constrains
 *  @args:	outfp (FILE *) -> output file descripter
 *  @return:	none
 */
void
declare_initial_constrains(FILE *outfp)
{
	fprintf(outfp, "##dummy constrain to satisfy the syntax of policy\n");
	fprintf(outfp, "constrain process transition\n( u1 == u2 or u1 != u2 );\n\n\n");
}




/**
 *  @name:	default_allow
 *  @about:	print default allow 
 *  @args:	outfp (FILE *) -> output file descripter
 *  @return:	none
 */
void
default_allow(FILE *outfp)
{
  fprintf(outfp,"##Following are allowed in simplifed policy by default\n");
  include_file(get_base_policy_files()->default_te, outfp);
}



static BASEPOLICY *base_files = NULL;
void set_base_policy_files(char *dir){
  if(base_files == NULL )
    {
      base_files = (BASEPOLICY *)malloc(sizeof(BASEPOLICY));
      if(base_files == NULL)
	{
	  perror("malloc");
	  exit(1);
	}
    }
  
  base_files->security_class = mk_fullpath(dir,SECURITY_CLASS_FILE);
  base_files->initial_sids = mk_fullpath(dir,INITIAL_SIDS_FILE);
  base_files->initial_sid_context = mk_fullpath(dir, INITIAL_SID_CONTEXT_FILE);
  base_files->fs_use = mk_fullpath(dir, FS_USE_FILE);
  base_files->genfs_context = mk_fullpath(dir, GENFS_CONTEXT_FILE);
  base_files->access_vectors = mk_fullpath(dir, ACCESS_VECTORS_FILE);
  base_files->default_te = mk_fullpath(dir, DEFAULT_TE_FILE);
  base_files->types_te = mk_fullpath(dir, TYPES_FILE);
  base_files->attribute_te = mk_fullpath(dir, ATTRIBUTE_FILE);
  
  return;
}

BASEPOLICY *get_base_policy_files(){
  return base_files;
}

void free_base_policy_files(){
  free(base_files);
}

